@@ -1,7 +1,7 @@
 /*
  * OpenConnect (SSL + DTLS) VPN client
  *
- * Copyright © 2008-2014 Intel Corporation.
+ * Copyright © 2008-2016 Intel Corporation.
  * Copyright © 2008 Nick Andrew <nick@nick-andrew.net>
  * Copyright © 2013 John Morrissey <jwm@horde.net>
  *
@@ -24,18 +24,55 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef _WIN32
 #define uid_t unsigned
 #endif
 
 #define OPENCONNECT_API_VERSION_MAJOR 5
-#define OPENCONNECT_API_VERSION_MINOR 1
+#define OPENCONNECT_API_VERSION_MINOR 5
 
 /*
- * API version 5.1:
- *  - Add openconnect_set_compression_mode().
+ * API version 5.5 (v8.00; 2019-01-05):
+ *  - add openconnect_set_version_string()
+ *  - add openconnect_set_key_password()
+ *  - Add openconnect_has_tss2_blob_support()
+ *  - Add openconnect_get_supported_protocols()
+ *  - Add openconnect_free_supported_protocols()
+ *  - Add openconnect_get_protocol()
+ *  - Add openconnect_get_idle_timeout()
  *
- * API version 5.0:
+ * API version 5.4 (v7.08; 2016-12-13):
+ *  - Add openconnect_set_pass_tos()
+ *
+ * API version 5.3 (v7.07; 2016-07-11):
+ *  - Add openconnect_set_localname().
+ *  - Add openconnect_override_getaddrinfo().
+ *  - Add openconnect_get_cstp_compression().
+ *  - Add openconnect_get_dtls_compression().
+ *  - Add openconnect_disable_ipv6().
+ *  - Add ip_info->gateway_addr.
+ *  - Add openconnect_set_setup_tun_handler().
+ *  - Add openconnect_set_reconnected_handler().
+ *  - Add openconnect_get_dnsname().
+ *  - Add openconnect_get_peer_cert_chain() and
+ *        openconnect_free_peer_cert_chain().
+ *
+ * API version 5.2 (v7.05; 2015-03-10):
+ *  - Add openconnect_set_http_auth(), openconnect_set_protocol().
+ *
+ * API version 5.1 (v7.05; 2015-03-10):
+ *  - Add openconnect_set_compression_mode(), openconnect_set_loglevel()
+ *
+ *   (Note: API 5.1 and openconnect_set_compression_mode() were present in
+ *    this file in the v7.04 release on 2015-01-25, but the symbol versioning
+ *    for the new function was OPENCONNECT_5_0, and openconnect_set_loglevel()
+ *    was not yet present.)
+ *
+ * API version 5.0 (v7.00; 2014-11-27):
  *  - Remove OPENCONNECT_X509 and openconnect_get_peer_cert().
  *  - Change openconnect_get_cert_der() to openconnect_get_peer_cert_DER() etc.
  *  - Add openconnect_check_peer_cert_hash().
@@ -43,27 +80,27 @@
  *  - Add openconnect_has_yubioath_support() and OC_TOKEN_MODE_YUBIOATH.
  *  - Add openconnect_has_system_key_support().
  *
- * API version 4.1:
+ * API version 4.1 (v7.00; 2014-11-27):
  *  - Add openconnect_get_cstp_cipher(), openconnect_get_dtls_cipher(),
  *    openconnect_set_system_trust(), openconnect_set_csd_environ().
  *  - Change openconnect_init_ssl() to return int.
  *
- * API version 4.0:
+ * API version 4.0 (v7.00; 2014-11-27):
  *  - Change string handling to never transfer ownership of allocations.
  *  - Add openconnect_set_option_value(), openconnect_free_cert_info().
  *
- * API version 3.4:
+ * API version 3.4 (v7.00; 2014-11-27):
  *  - Add openconnect_set_token_callbacks()
  *
- * API version 3.3:
+ * API version 3.3 (v6.00; 2014-07-08):
  *  - Add openconnect_set_pfs(), openconnect_set_dpd(),
  *    openconnect_set_proxy_auth()
  *
- * API version 3.2:
+ * API version 3.2 (v5.99; 2014-03-05):
  *  - Add OC_TOKEN_MODE_HOTP and allow openconnect_has_oath_support() to
  *    return 2 to indicate that it is present.
  *
- * API version 3.1:
+ * API version 3.1 (v5.99; 2014-03-05):
  *  - Add openconnect_setup_cmd_pipe(), openconnect_mainloop(),
  *    openconnect_setup_tun_device(), openconnect_setup_tun_script(),
  *    openconnect_setup_tun_fd(), openconnect_setup_dtls(),
@@ -71,44 +108,44 @@
  *    openconnect_get_ifname(), openconnect_set_reqmtu(),
  *    openconnect_get_ip_info(), openconnect_set_protect_socket_handler(),
  *    openconnect_set_mobile_info(), openconnect_set_xmlpost(),
-7 *    openconnect_set_stats_handler()
+ *    openconnect_set_stats_handler()
  *
- * API version 3.0:
+ * API version 3.0 (v5.99; 2014-03-05):
  *  - Change oc_form_opt_select->choices to an array of pointers
  *  - Add oc_form_opt->flags
  *  - Add OC_FORM_RESULT_* and oc_auth_form->authgroup_*
  *
- * API version 2.2:
+ * API version 2.2 (v5.00; 2013-05-16):
  *  - Add openconnect_set_token_mode(), openconnect_has_oath_support()
  *  - Deprecate openconnect_set_stoken_mode()
  *
- * API version 2.1:
+ * API version 2.1 (v4.99; 2013-02-07):
  *  - Add openconnect_set_reported_os()
  *  - Add openconnect_set_stoken_mode(), openconnect_has_stoken_support()
  *
- * API version 2.0:
+ * API version 2.0 (v3.99; 2012-06-13):
  *  - OPENCONNECT_X509 is now an opaque type.
  *  - Add openconnect_has_pkcs11_support(), openconnect_has_tss_blob_support()
  *  - Rename openconnect_init_openssl() -> openconnect_init_ssl()
  *  - Rename openconnect_vpninfo_new_with_cbdata() -> openconnect_vpninfo_new()
  *    and kill the old openconnect_vpninfo_new() and its callback types.
  *
- * API version 1.5:
+ * API version 1.5 (v3.99; 2012-06-13):
  *  - Add openconnect_get_cert_details(), openconnect_get_cert_DER().
  *
- * API version 1.4:
+ * API version 1.4 (v3.19; 2012-05-17):
  *  - Add openconnect_set_cancel_fd()
  *
- * API version 1.3:
+ * API version 1.3 (v3.13; 2011-09-30):
  *  - Add openconnect_set_cert_expiry_warning() to change from default 60 days
  *
- * API version 1.2:
+ * API version 1.2 (v3.10; 2011-06-30)
  *  - Add openconnect_vpninfo_new_with_cbdata()
  *
- * API version 1.1:
+ * API version 1.1 (v3.02; 2011-04-19):
  *  - Add openconnect_vpninfo_free()
  *
- * API version 1.0:
+ * API version 1.0 (v3.00; 2011-03-09):
  *  - Initial version
  *
  * NEW LIBRARY FUNCTION CHECKLIST:
@@ -119,7 +156,7 @@
  * 3) Add function to libopenconnect.map.in
  * 4) Add declaration + comments in the latter part of this file
  * 5) Add function to jni.c, then test with ./configure --with-java && make
- * 6) Add declaration to LibOpenConnect.java, then run "ant" to test
+ * 6) Add declaration to LibOpenConnect.java, then run "cd java && ant" to test
  */
 
 /* Before API version 1.4 (OpenConnect 3.19) this macro didn't exist.
@@ -138,6 +175,23 @@
 
 /****************************************************************************/
 
+/* Enumeration of supported VPN protocols */
+
+#define OC_PROTO_PROXY	(1<<0)
+#define OC_PROTO_CSD	(1<<1)
+#define OC_PROTO_AUTH_CERT	(1<<2)
+#define OC_PROTO_AUTH_OTP	(1<<3)
+#define OC_PROTO_AUTH_STOKEN	(1<<4)
+
+struct oc_vpn_proto {
+	const char *name;
+	const char *pretty_name;
+	const char *description;
+	unsigned int flags;
+};
+
+/****************************************************************************/
+
 /* Authentication form processing */
 
 #define OC_FORM_OPT_TEXT	1
@@ -212,7 +266,7 @@ struct oc_auth_form {
 };
 
 struct oc_split_include {
-	char *route;
+	const char *route;
 	struct oc_split_include *next;
 };
 
@@ -230,6 +284,11 @@ struct oc_ip_info {
 	struct oc_split_include *split_dns;
 	struct oc_split_include *split_includes;
 	struct oc_split_include *split_excludes;
+
+	/* The elements above this line come from server-provided CSTP headers,
+	 * so they should be handled with caution.  gateway_addr is generated
+	 * locally from getnameinfo(). */
+	char *gateway_addr;
 };
 
 struct oc_vpn_option {
@@ -245,6 +304,12 @@ struct oc_stats {
 	uint64_t rx_bytes;
 };
 
+struct oc_cert {
+	int der_len;
+	unsigned char *der_data;
+	void *reserved;
+};
+
 /****************************************************************************/
 
 #define PRG_ERR		0
@@ -339,8 +404,20 @@ int openconnect_get_peer_cert_DER(struct openconnect_info *vpninfo,
 				  unsigned char **buf);
 void openconnect_free_cert_info(struct openconnect_info *vpninfo,
 				void *buf);
+
+/* Creates a list of all certs in the peer's chain, returning the
+   number of certs in the chain (or <0 on error). Only valid inside the
+   validate_peer_cert callback. The caller should free the chain,
+   but should not modify the contents. */
+int openconnect_get_peer_cert_chain(struct openconnect_info *vpninfo,
+				    struct oc_cert **chain);
+void openconnect_free_peer_cert_chain(struct openconnect_info *vpninfo,
+				      struct oc_cert *chain);
+
 /* Contains a comma-separated list of authentication methods to enabled.
    Currently supported: Negotiate,NTLM,Digest,Basic */
+int openconnect_set_http_auth(struct openconnect_info *vpninfo,
+			      const char *methods);
 int openconnect_set_proxy_auth(struct openconnect_info *vpninfo,
 			       const char *methods);
 int openconnect_set_http_proxy(struct openconnect_info *vpninfo,
@@ -357,10 +434,29 @@ int openconnect_init_ssl(void);
 const char *openconnect_get_cstp_cipher(struct openconnect_info *);
 const char *openconnect_get_dtls_cipher(struct openconnect_info *);
 
+/* These return a descriptive string of the compression algorithm 
+ * in use (LZS, LZ4, ...). If no compression then NULL is returned. */
+const char *openconnect_get_cstp_compression(struct openconnect_info *);
+const char *openconnect_get_dtls_compression(struct openconnect_info *);
+
+/* Returns the IP address of the exact host to which the connection
+ * was made. In --cookieonly mode or in any other scenario involving
+ * a "two stage" connection, it is important to reconnect by IP because
+ * the server side may be using DNS trickery for load balancing.
+ *
+ * If the IP address is unavailable due to the use of a proxy, this will
+ * fall back to returning the DNS name. */
 const char *openconnect_get_hostname(struct openconnect_info *);
+
+/* Returns the hostname parsed out of the server name URL. This is
+ * intended to be used by the validate_peer_cert callback to check that
+ * the certificate matches the server name. */
+const char *openconnect_get_dnsname(struct openconnect_info *);
+
 int openconnect_set_hostname(struct openconnect_info *, const char *);
 char *openconnect_get_urlpath(struct openconnect_info *);
 int openconnect_set_urlpath(struct openconnect_info *, const char *);
+int openconnect_set_localname(struct openconnect_info *, const char *);
 
 /* Some software tokens, such as HOTP tokens, include a counter which
  * needs to be stored in persistent storage.
@@ -413,15 +509,19 @@ void openconnect_set_xmlpost(struct openconnect_info *, int enable);
    trojan binary. */
 int openconnect_set_reported_os(struct openconnect_info *, const char *os);
 
+int openconnect_set_version_string(struct openconnect_info *vpninfo,
+				   const char *version_string);
 int openconnect_set_mobile_info(struct openconnect_info *vpninfo,
 				const char *mobile_platform_version,
 				const char *mobile_device_type,
 				const char *mobile_device_uniqueid);
 int openconnect_set_client_cert(struct openconnect_info *, const char *cert,
 				const char *sslkey);
+int openconnect_set_key_password(struct openconnect_info *vpninfo, const char *pass);
 const char *openconnect_get_ifname(struct openconnect_info *);
 void openconnect_set_reqmtu(struct openconnect_info *, int reqmtu);
 void openconnect_set_dpd(struct openconnect_info *, int min_seconds);
+int openconnect_get_idle_timeout(struct openconnect_info *);
 
 /* The returned structures are owned by the library and may be freed/replaced
    due to rekey or reconnect. Assume that once the mainloop starts, the
@@ -436,6 +536,7 @@ int openconnect_get_port(struct openconnect_info *);
 const char *openconnect_get_cookie(struct openconnect_info *);
 void openconnect_clear_cookie(struct openconnect_info *);
 
+void openconnect_disable_ipv6(struct openconnect_info *vpninfo);
 void openconnect_reset_ssl(struct openconnect_info *vpninfo);
 int openconnect_parse_url(struct openconnect_info *vpninfo, const char *url);
 void openconnect_set_cert_expiry_warning(struct openconnect_info *vpninfo,
@@ -544,6 +645,10 @@ typedef void (*openconnect_protect_socket_vfn) (void *privdata, int fd);
 void openconnect_set_protect_socket_handler(struct openconnect_info *vpninfo,
 					    openconnect_protect_socket_vfn protect_socket);
 
+void openconnect_set_loglevel(struct openconnect_info *vpninfo, int level);
+
+void openconnect_set_pass_tos(struct openconnect_info *vpninfo, int enable);
+
 /* Callback for obtaining traffic stats via OC_CMD_STATS.
  */
 typedef void (*openconnect_stats_vfn) (void *privdata, const struct oc_stats *stats);
@@ -555,9 +660,9 @@ void openconnect_set_stats_handler(struct openconnect_info *vpninfo,
 int openconnect_has_pkcs11_support(void);
 
 /* The OpenSSL TPM ENGINE stores keys in a PEM file labelled with the string
-   -----BEGIN TSS KEY BLOB-----. GnuTLS may learn to support this format too,
-   in the near future. */
+   -----BEGIN TSS KEY BLOB-----. */
 int openconnect_has_tss_blob_support(void);
+int openconnect_has_tss2_blob_support(void);
 
 /* Software token capabilities. */
 int openconnect_has_stoken_support(void);
@@ -565,4 +670,29 @@ int openconnect_has_oath_support(void);
 int openconnect_has_yubioath_support(void);
 int openconnect_has_system_key_support(void);
 
+/* Query and select from among supported protocols */
+int openconnect_get_supported_protocols(struct oc_vpn_proto **protos);
+void openconnect_free_supported_protocols(struct oc_vpn_proto *protos);
+const char *openconnect_get_protocol(struct openconnect_info *vpninfo);
+int openconnect_set_protocol(struct openconnect_info *vpninfo, const char *protocol);
+
+struct addrinfo;
+typedef int (*openconnect_getaddrinfo_vfn) (void *privdata, const char *node, const char *service,
+					    const struct addrinfo *hints, struct addrinfo **res);
+void openconnect_override_getaddrinfo(struct openconnect_info *vpninfo, openconnect_getaddrinfo_vfn gai_fn);
+
+/* Callback for configuring the interface after MTU detection finishes. */
+typedef void (*openconnect_setup_tun_vfn) (void *privdata);
+void openconnect_set_setup_tun_handler(struct openconnect_info *vpninfo,
+				       openconnect_setup_tun_vfn setup_tun);
+
+/* Callback for indicating that a TCP reconnection succeeded. */
+typedef void (*openconnect_reconnected_vfn) (void *privdata);
+void openconnect_set_reconnected_handler(struct openconnect_info *vpninfo,
+				         openconnect_reconnected_vfn reconnected_fn);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* __OPENCONNECT_H__ */
